/* thermometer_display
 *
 * Hőmérés MCP9700 hőmérővel. Az A4 analóg bementre
 * kötött MCP9700 hőmérő jelét a beépített 1.1 V-os referenciához
 * képest mérjük, 50-szer ismételve, s az eredményt átlagoljuk.
 * Az átlagolt eredményt Celsius fokokra átszámítjuk és a
 * Multifunkciós kártya hétszegmenses kijelzőére kiíratjuk.
 * Az eredményeket ellenőrzés gyanánt soros porton is kiíratjuk
 * (a mért feszültséget és az átszámított eredményt is).
 * A kiírások között ötmásodperces szünetet tartunk.
 *
 * Az MCP9700 hőmérő nullapontja = 500 mV, érzékenysége = 10 mV/C
 */

/* Define shift register pins used for seven segment display */
#define LATCH_DIO 4
#define CLK_DIO 7
#define DATA_DIO 8

/* Segment byte maps for numbers 0 to 9 */
const byte SEGMENT_MAP[] = {0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0X80, 0X90};
/* Byte maps to select digit 1 to 4 */
const byte SEGMENT_SELECT[] = {0xF1, 0xF2, 0xF4, 0xF8};
byte segment_data[] = {0xFF, 0xFF, 0x9C, 0xC6};
byte counter = 50;
byte idx = 0;
long nexttime;
long sum = 0;

void setup () {
  Serial.begin(115200);
  /* Set DIO pins to outputs */
  pinMode(LATCH_DIO, OUTPUT);
  pinMode(CLK_DIO, OUTPUT);
  pinMode(DATA_DIO, OUTPUT);
  nexttime = millis();
}

/* Main program */
void loop() {
  long voltage, tempC;
  if (millis() > nexttime) {          // TASK1 100 ms-onként fut le
    sum += analogRead(A4);
    if (--counter == 0) {
      voltage = sum * 100 / 1024;    // Feszültség mV-ban
      tempC = (voltage - 500) / 10;  // Hőmérséklet Celsius fokokban
      sum = 0;                       // Töröljük az összegző változót
      counter = 50;                  // 50 konverzió eredményét átlagoljuk
      segment_data[0] = SEGMENT_MAP[tempC / 10];
      segment_data[1] = SEGMENT_MAP[tempC % 10];
      float temp = (voltage - 500) / 10.0;
      Serial.print("Voltage: ");
      Serial.print(voltage);
      Serial.print(" mV  Temp: ");
      Serial.print(temp, 1);
      Serial.println(" ºC");
    }
    nexttime += 100;                 // 100 ms-onként végzünk konverziót
  }
  /* Mérési szünetekben pörgetjük a kijelzést (TASK2) */
  refresh_display();
}

/* A soron következő számjegy megjelenítése */
void refresh_display() {
  byte i;
  i = idx++ & 0x3;
  digitalWrite(LATCH_DIO, LOW);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, segment_data[i]);
  shiftOut(DATA_DIO, CLK_DIO, MSBFIRST, SEGMENT_SELECT[i] );
  digitalWrite(LATCH_DIO, HIGH);
}
